## À exécuter une fois pour toute
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import os

# changement du répertoire de travail. Modifier de façon à indiquer le répertoire où se trouvent les images.
os.chdir('/home/formation/python/')

## Exemples pour le chargement et l'affichage d'une image

# charge une image sous forme de liste de listes
image = (mpimg.imread('./guepiersNB.jpg')).tolist()
# affichage des dimensions de l'image
print('w=', len(image[0]),' h=', len(image))
# affichage de l'image
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()

## Pour se chauffer...

def dim(img:list) -> tuple:
    '''Renvoie le couple (largeur,hauteur) de l'image'''
    return (len(img[0]), len(img))

def contraste_inverse(img:list) -> None:
    '''Inverse le contraste de img. Effet de bord : l'image est modifiée'''
    (w, h) = dim(img)
    for i in range(h):
        for j in range(w):
            img[i][j] = 255 - img[i][j]

image = (mpimg.imread('./flamantsNB.jpg')).tolist()
contraste_inverse(image)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()

def seuil(img:list, s:int) -> None:
    '''Seuille l'image img : tout pixel d'intensite < s devient noir, tout autre pixel devient blanc. Effet de bord : l'image est modifiée'''
    (w, h) = dim(img)
    for i in range(h):
        for j in range(w):
            if img[i][j] < s:
                img[i][j] = 0
            else:
                img[i][j] = 255


image = (mpimg.imread('./flamantsNB.jpg')).tolist()
seuil(image, 128)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()


def flip(img:list) -> None:
    '''Effectue une symétrie de l'image. Effet de bord : l'image est modifiée'''
    (w, h) = dim(img)
    for i in range(h):
        for j in range(w//2):
            (img[i][j], img[i][len(img[0])-1-j]) = (img[i][len(img[0])-1-j], img[i][j]) 

image = (mpimg.imread('./flamantsNB.jpg')).tolist()
flip(image)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()



## Réduction de moitié de la largeur (version naïve)

def reduction_moitie_ligne(l:list) -> list:
    '''Moyenne deux à deux les éléments successifs d'une liste l d'entiers de longueur 2*n. Renvoie une nouvelle liste d'entiers de longueur n'''
    r = []
    for i in range(len(l)//2):
        r.append((l[2*i]+l[2*i+1])//2)
    return r
    
def reduction_moitie_image(img:list) -> None:
    '''Réduit de moitié une image dans le sens de la largeur. Effet de bord : l'image est modifiée'''
    for i in range(len(img)):
        img[i] = reduction_moitie_ligne(img[i])

image = (mpimg.imread('./flamantsNB.jpg')).tolist()
reduction_moitie_image(image)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()

## Calcul de l'énergie

def energie(img:list) -> list:
    '''Calcule l'énergie d'une image img.'''
    (w, h) = dim(img)
    e = [[0 for j in range(w)] for i in range(h)]
    for i in range(h):
        for j in range(w):
            if i == 0:
                vx = img[i+1][j] - img[i][j]
            elif i == len(e) - 1:
                vx = img[i][j] - img[i-1][j]
            else:
                vx = (img[i+1][j] - img[i-1][j])/2
            if j == 0:
                vy = img[i][j+1] - img[i][j]
            elif j == len(e[0]) - 1:
                vy = img[i][j] - img[i][j-1]
            else:
                vy = (img[i][j+1] - img[i][j-1])/2
            e[i][j] = abs(vx)+abs(vy)
    return e
    
        

image = (mpimg.imread('./guepiersNB.jpg')).tolist()
energie_image = energie(image)
plt.imshow(energie_image,cmap='gray',clim=(0,100)) #127 pour augmenter le contraste.
plt.show()
      
## Manipulations de listes

def enlever(l:list,i:int) -> list:
    '''Supprime l'élément d'indice i de la liste l. Renvoie une nouvelle liste.'''
    return l[:i]+l[i+1:]
    
def indice_min(l:list) -> int:
    '''Renvoie un indice où apparaît le minimum d'une liste non vide de nombres l.'''
    m = 0
    for i in range(len(l)):
        if l[i] < l[m]:
            m = i
    return m
    
## Réduction ligne par ligne

def reduction_ligne_par_ligne(img:list) -> None:
    '''Enlève le pixel d'énergie minimale de chaque ligne de l'image img. Effet de bord : modifie l'image'''
    e = energie(img)
    for i in range(len(img)):
        m = indice_min(e[i])
        img[i] = enlever(img[i], m)


def itere_reduction_ligne_par_ligne(img:list, n:int) -> None:
    '''Réduit de n pixel de largeur l'image img. Effet de bord : modifie l'image.'''
    for _ in range(n):
        reduction_ligne_par_ligne(img)
        
image = (mpimg.imread('./flamantsNB.jpg')).tolist()
itere_reduction_ligne_par_ligne(image,200)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()

## Réduction par colonne

def meilleure_colonne(e:list) -> int:
    '''Renvoie l'indice d'une colonne d'énergie minimale, pour une image d'énergies e'''
    (w, h) = dim(e)
    energie_colonnes = [0 for _ in range(w)]
    for i in range(h):
        for j in range(w):
            energie_colonnes[j] += e[i][j]
    return indice_min(energie_colonnes)
    
def reduction_meilleure_colonne(img:list) -> None:
    '''Enlève une colonne d'énergie minimale dans une image img. Effet de bord : modifie l'image'''
    r = []
    e = energie(img)
    mc = meilleure_colonne(e)
    for i in range(len(img)):
        l = img[i]
        img[i]=enlever(img[i], mc)
    
def itere_reduction_meilleure_colonne(img:list, n:int) -> None:
    for _ in range(n):
        reduction_meilleure_colonne(img)
    
image = (mpimg.imread('./rueNB.jpg')).tolist()
itere_reduction_meilleure_colonne(image,100)
plt.imshow(image,cmap='gray',clim=(0,255))
plt.show()

## Seam carving


def energie_chemins_partiels(e:list) -> list:
    '''Calcule l'image des énergies des meilleurs chemins partiels du haut de l'image jusqu'à chaque pixel. e est une image des énergies.'''
    E = [e[0]]
    for i in range(1, len(e)):
        E.append([e[i][j] + min(E[i-1][max(j-1,0):j+2]) for j in range(len(e[0]))])
    return E

e = [[1, 1, 0, 3],
     [4, 1, 2, 4],
     [1, 2, 2, 1],
     [4, 1, 1, 0]]

print(energie_chemins_partiels(e))

def indice_min_entre(l:list, d:int, f:int) -> int:
    '''Renvoie un indice où apparaît le minimum d'une liste non vide de nombres l entre les indices d (inclus) et f (exclu).'''
    if d < 0:
        d = 0
    if f > len(l):
        f = len(l)
    m = d
    for i in range(d+1, f):
        if l[i] < l[m]:
            m = i
    return m


def chemin_minimal(e:list) -> list:
    '''Calcule un chemin d'énergie minimale dans une image des énergies e. Le résultat renvoyé est la suite des abscisses des pixels du chemin, de bas en haut'''
    E = energie_chemins_partiels(e)
    r = [indice_min(E[-1])]
    for i in range(len(e)-2, -1, -1):
        r.append(indice_min_entre(E[i],r[-1]-1,r[-1]+2)) # cherche le pixel d'énergie minimale parmi les 3 (ou 2 aux bords) voisins de (i+1,r[-1]) sur la ligne i
    return r
     
print(chemin_minimal(e))

def seam_carving(img:list, n:int) -> None: 
    '''applique n fois le retrait de chemin minimal à l'image img. Effet de bord : modifie l'image'''
    j = 0
    for _ in range(n):
        j += 1
        e = energie(img)
        E = energie_chemins_partiels(e)
        c = chemin_minimal(E)
        print(j)
        if j%10 == 0:
            print(c)
        for i in range(len(img)):
            img[i] = enlever(img[i],c[len(img)-1-i])

image = (mpimg.imread('./guepiersNB.jpg')).tolist()
seam_carving(image, 150)
plt.imshow(image,cmap='gray',clim = (0, 255))
plt.show()
